using System;
using System.Collections.Generic;
using System.Text;
using System.Timers;
using System.Data.OleDb;
using System.Data;

namespace ComponenteActivo
{
    internal class ChamadaImpl : IChamada
    {
        // possui estado
        Timer myTimer=null;
        string CONN = @"Provider=Microsoft.Jet.OLEDB.4.0;Data Source=..//..//..//db.mdb";


        // Processo de incremento de tempo nas chamadas activas
        private void myTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            OleDbConnection conn = new OleDbConnection(CONN);
            int n=0;

            // Actualiza todas as chamadas
            try
            {
                conn.Open();
                OleDbCommand cmd = new OleDbCommand("UPDATE Chamadas SET segundosDecorridos = segundosDecorridos + 1 WHERE activa = Yes", conn);
                n = cmd.ExecuteNonQuery();
            }
            catch
            {
                myTimer.Enabled = false;
                myTimer = null;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            // Pra actualizaes
            if (n == 0)
            {
                myTimer.Enabled = false;
                myTimer = null;
            }
        }



        #region IChamada Members

        public ChamadaStatus GetSegundos(string numeroIniciador, out int nsegundos)
        {
            nsegundos = -1;
            if ((numeroIniciador == null) || (numeroIniciador == "")) return ChamadaStatus.PARAMETER_PROBLEM;

            OleDbConnection conn = new OleDbConnection(CONN);

            try
            {
                conn.Open();
                // WHERE numeroIniciador = ?
                OleDbCommand cmd = new OleDbCommand("SELECT segundosDecorridos FROM Chamadas WHERE numeroIniciador = ? AND activa = Yes", conn);
                cmd.Parameters.AddWithValue("iniciador", numeroIniciador);

                object obj  = cmd.ExecuteScalar();
                if (obj == null) return ChamadaStatus.INEXISTENT_INITIATOR;
                nsegundos = (int)obj;
            }
            catch
            {
                return ChamadaStatus.DATABASE_PROBLEM;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            return ChamadaStatus.OK;
        }

        public ChamadaStatus Start(string numeroIniciador, string numeroChamado, DateTime instanteInicio)
        {
            if ((numeroIniciador==null) || (numeroIniciador == "") || 
               (numeroChamado == null) || (numeroChamado == "") ||
               (instanteInicio == null)) return ChamadaStatus.PARAMETER_PROBLEM;

            OleDbConnection conn = new OleDbConnection(CONN);

            // Chamada j existente?
            try
            {
                conn.Open();
                OleDbCommand cmd = new OleDbCommand("SELECT activa FROM Chamadas WHERE activa = Yes AND (numeroIniciador = ? OR numeroIniciador = ? OR numeroChamado = ? OR numeroChamado = ? )", conn);
                cmd.Parameters.AddWithValue("inicia1", numeroIniciador);
                cmd.Parameters.AddWithValue("inicia2", numeroChamado);
                cmd.Parameters.AddWithValue("chamado1", numeroChamado);
                cmd.Parameters.AddWithValue("chamado2", numeroIniciador);
                object obj = cmd.ExecuteScalar();
                if (obj != null) return ChamadaStatus.ALREADY_INITIATED;

                cmd = new OleDbCommand("INSERT INTO Chamadas (instanteInicio, segundosDecorridos, activa, numeroIniciador, numeroChamado) VALUES (?,0,Yes,?,?)", conn);
                //cmd = new OleDbCommand("INSERT INTO Chamadas (instanteInicio) VALUES (?)", conn);
                cmd.Parameters.AddWithValue("@instante", instanteInicio).OleDbType = OleDbType.Date; // Access Date/Time = .NET OleDbType.Date
                cmd.Parameters.AddWithValue("iniciador", numeroIniciador);
                cmd.Parameters.AddWithValue("chamado", numeroChamado);

                cmd.ExecuteNonQuery();
            }
            catch
            {
                return ChamadaStatus.DATABASE_PROBLEM;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            // Inicia o processo de Timer se nunca foi chamado
            if (myTimer == null)
            {
                myTimer = new Timer(1000); // 1 segundo
                myTimer.AutoReset = true;  // reset the timer
                myTimer.Elapsed +=          // regista handler
                new System.Timers.ElapsedEventHandler(myTimer_Elapsed);
                myTimer.Start();
            }

            return ChamadaStatus.OK;
        }

        public ChamadaStatus Stop(string numeroIniciador)
        {
            if ((numeroIniciador == null) || (numeroIniciador == "")) return ChamadaStatus.PARAMETER_PROBLEM;
            
            OleDbConnection conn = new OleDbConnection(CONN);

            // Chamada j existente?
            try
            {
                conn.Open();
                // WHERE numeroIniciador = ?
                OleDbCommand cmd = new OleDbCommand("UPDATE Chamadas SET activa = No WHERE numeroIniciador = ? AND activa = Yes", conn);
                cmd.Parameters.AddWithValue("iniciador", numeroIniciador);

                int n =cmd.ExecuteNonQuery();
                if (n < 1) return ChamadaStatus.NOT_INITIATED;
                if (n > 1) return ChamadaStatus.MULTIPLE_CALLS;
            }
            catch
            {
                return ChamadaStatus.DATABASE_PROBLEM;
            }
            finally
            {
                if (conn.State == System.Data.ConnectionState.Open)
                    conn.Close();
            }

            return ChamadaStatus.OK;
        }

        public ChamadaStatus GetChamadasActivas(out DataSet chamadasActivas)
        {
            chamadasActivas = new DataSet();
            OleDbDataAdapter da = new OleDbDataAdapter("SELECT numeroIniciador, numeroChamado, instanteInicio, segundosDecorridos FROM Chamadas WHERE activa = Yes", CONN);
            try
            {
                da.Fill(chamadasActivas);
            }
            catch
            {
                return ChamadaStatus.DATABASE_PROBLEM;
            }
            return ChamadaStatus.OK;
        }


        #endregion
    }
}
